package net.wanji.common.tool.wsdltool;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.http.HttpEntity;
import org.apache.http.ParseException;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.Charset;

/**
 * @ClassName DoSoapTool
 * @Description
 * @Author mapabc7
 * @Date 2020/7/28 10:30
 * @Version 1.0
 */
public class DoSoapTool {
    public final static String charset = "UTF-8";

    /**
     * @param address
     * @param charset
     * @param requestXml
     * @throws MalformedURLException
     * @throws IOException
     */
    @Deprecated
    private static String doPostSoap_(String address, String charset, String requestXml) throws MalformedURLException, IOException {
        HttpURLConnection httpURLConnection = null;
        try {
            httpURLConnection = (HttpURLConnection) new URL(address).openConnection();
            httpURLConnection.setRequestMethod("POST");
            httpURLConnection.setDoOutput(true);
            httpURLConnection.setDoInput(true);
            httpURLConnection.setRequestProperty("Content-Type", "text/xml;charset=" + charset);
            OutputStream outputStream = httpURLConnection.getOutputStream();
            outputStream.write(requestXml.getBytes(charset));
            int responseCode = httpURLConnection.getResponseCode();
            StringBuilder stringBuffer = new StringBuilder();
            if (HttpStatus.SC_OK == responseCode) {
                InputStream inputStream = httpURLConnection.getInputStream();
                byte[] buffer = new byte[1024];
                int len = 0;
                while ((len = inputStream.read(buffer)) > 0) {
                    stringBuffer.append(new String(buffer, 0, len));
                }
            }
            return stringBuffer.toString();
        } finally {
            if (null != httpURLConnection) httpURLConnection.disconnect();
        }
    }

    public static String doPostSoap(String address, String requestXml) throws ParseException, IOException {
        return doPostSoap(address, requestXml, charset);
    }

    public static String doPostSoap(String address, String requestXml, String charset) throws ParseException, IOException {
        return doPostSoap(address, charset, requestXml, null, null, false);
    }

    public static String doPostSoap(String address, String requestXml, String charset, String username, String password) throws ParseException, IOException {
        return doPostSoap(address, charset, requestXml, username, password, true);
    }

    public static String doPostSoap(String address, String requestXml, String username, String password) throws ParseException, IOException {
        return doPostSoap(address, requestXml, charset, username, password);
    }

    /**
     * @param url
     * @param charset
     * @param requestXml
     * @param username
     * @param password
     * @param auth
     * @return
     * @throws ParseException
     * @throws IOException
     */
    public static String doPostSoap(String url, String charset, String requestXml, String username,
                                    String password, boolean auth) throws ParseException, IOException {
        CloseableHttpClient httpClient = null;
        try {
            HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
            httpClient = httpClientBuilder.build();
            HttpPost httpPost = new HttpPost(url);
            RequestConfig requestConfig = RequestConfig.custom().setSocketTimeout(60 * 1000)
                    .setConnectTimeout(60 * 1000).build();
            httpPost.setConfig(requestConfig);
            httpPost.setHeader("Content-Type", "text/xml;charset=" + charset);
            //httpPost.setHeader("SOAPAction", methodName);
            if (auth)
                httpPost.setHeader("Authorization", authHeader(username, password, charset));
            StringEntity data = new StringEntity(requestXml, Charset.forName(charset));
            httpPost.setEntity(data);
            CloseableHttpResponse response = httpClient.execute(httpPost);
            if (HttpStatus.SC_OK == response.getStatusLine().getStatusCode()) {
                HttpEntity httpEntity = response.getEntity();
                if (httpEntity != null) {
                    return EntityUtils.toString(httpEntity, charset);
                }
            }
        } finally {
            // 释放资源
            if (httpClient != null) httpClient.close();
        }
        return null;
    }

    private static String authHeader(String userName, String password, String charset) {
        String auth = userName + ":" + password;
        byte[] encodedAuth = Base64.encodeBase64(auth.getBytes(Charset.forName(charset)));
        return "Basic " + new String(encodedAuth);
    }

    public static void main(String[] args) throws IOException {
        String reqXML = "<soap:Envelope xmlns:soap=\"http://www.w3.org/2003/05/soap-envelope\" xmlns:web=\"http://WebXml.com.cn/\">\n" +
                "   <soap:Header/>\n" +
                "   <soap:Body>\n" +
                "      <web:getSupportCity>\n" +
                "         <!--Optional:-->\n" +
                "         <web:byProvinceName>河北</web:byProvinceName>\n" +
                "      </web:getSupportCity>\n" +
                "   </soap:Body>\n" +
                "</soap:Envelope>";
        String result = DoSoapTool.doPostSoap("http://www.webxml.com.cn/WebServices/WeatherWebService.asmx", reqXML);
        System.out.println(result);
    }
}
